local GameObject = Object:extend()

function GameObject:new(area, x, y, opts)
    local opts = opts or {}
    if opts then for k, v in pairs(opts) do self[k] = v end end
    self.area = area
    self.x, self.y = x, y
    self.id = getUID(opts.id)
    self.dead = false
    self.timer = Timer()
end

function GameObject:update(dt)
    if self.timer then self.timer:update(dt) end
    if self.collider then self.x, self.y = self.body:getPosition() end
end

function GameObject:draw()

end

function GameObject:destroy()
    self.timer:destroy()
    self:removePhysics()
end

function GameObject:addPhysics(x, y, opts)
    local collider_opts = {collision_class = opts.collision_class, body_type = opts.body_type, game_object = self}
    if opts.collider_type then
        if opts.collider_type == 'Circle' then self.collider = self.area.world:newCircleCollider(x, y, opts.r, collider_opts)
        elseif opts.collider_type == 'Rectangle' then self.collider = self.area.world:newRectangleCollider(x, y, opts.w, opts.h, collider_opts)
        elseif opts.collider_type == 'BSGRectangle' then self.collider = self.area.world:newBSGRectangleCollider(x, y, opts.w, opts.h, opts.s, collider_opts)
        elseif opts.collider_type == 'Polygon' then self.collider = self.area.world:newPolygonCollider(opts.vertices, collider_opts)
        elseif opts.collider_type == 'Line' then self.collider = self.area.world:newLineCollider(opts.x1, opts.y1, opts.x2, opts.y2, collider_opts)
        elseif opts.collider_type == 'Chain' then self.collider = self.area.world:newChainCollider(opts.vertices, opts.loop, collider_opts) end
    else self.collider = self.area.world:newRectangleCollider(x, y, opts.w, opts.h, collider_opts) end
    self.body = self.collider.body
    self.collider.fixtures.main:setFriction(0)
end

function GameObject:removePhysics()
    if self.collider then
        self.collider:destroy()
        self.collider = nil
        self.body = nil
    end
end


return GameObject
