local Character = {}
Character.__index = Character

function Character.new(text, character, character_index, line_index, line, text_settings)
    local self = {}

    self.dt = 0
    self.text = text
    self.text_settings = text_settings
    self.x, self.y = 0, 0
    self.r, self.sx, self.sy, self.ox, self.oy = 0, 1, 1, 0, 0
    self.base_x, self.base_y = text.x, text.y
    self.font = text_settings.font or text.font or love.graphics.getFont()
    self.index = character_index
    self.line_index = line_index
    self.line = line
    self.character = character
    self.w = self.font:getWidth(self.character)
    self.h = self.font:getHeight()
    self.init_functions = text_settings.init or {}
    self.update_functions = text_settings.update or {}
    self.values = text_settings.values or {}
    for _, init_function in ipairs(self.init_functions) do init_function(self) end

    return setmetatable(self, Character)
end

function Character:update(dt)
    self.dt = dt
end

function Character:draw(ox, oy)
    love.graphics.setFont(self.font)
    for _, update_function in ipairs(self.update_functions) do update_function(self) end
    love.graphics.print(self.character, self.x, self.y, self.r, self.sx, self.sy, self.ox + ox, self.oy + oy)  
    love.graphics.setColor(255, 255, 255, 255)
end

function Character:setPosition()
    local x = self.text:getLineWidthUpToIndex(self.line, self.line_index)
    self.x = self.base_x + x
    local y = self.text:getLineHeightUpToLine(self.line)
    self.y = self.base_y + y
end

function Character:getWidth()
    return self.font:getWidth(self.character)
end

function Character:getHeight()
    return self.font:getHeight()
end

return setmetatable({new = new}, {__call = function(_, ...) return Character.new(...) end})
