local ImageCharacter = {}
ImageCharacter.__index = ImageCharacter

function ImageCharacter.new(text, image, character_index, line_index, line, text_settings)
    local self = {}

    self.text = text
    self.text_settings = text_settings
    self.x, self.y = 0, 0
    self.r, self.sx, self.sy, self.ox, self.oy = 0, 1, 1, 0, 0
    self.base_x, self.base_y = text.x, text.y
    self.index = character_index
    self.line_index = line_index
    self.line = line
    self.image = image 
    self.w = self.image:getWidth()
    self.h = self.image:getHeight()
    self.init_functions = text_settings.init or {}
    self.update_functions = text_settings.update or {}
    for _, init_function in ipairs(self.init_functions) do init_function(self) end

    return setmetatable(self, ImageCharacter)
end

function ImageCharacter:update(dt)
    self.dt = dt
end

function ImageCharacter:draw()
    for _, update_function in ipairs(self.update_functions) do update_function(self) end
    love.graphics.draw(self.image, self.x, self.y, self.r, self.sx, self.sy, self.ox, self.oy) 
    love.graphics.setColor(255, 255, 255, 255)
end

function ImageCharacter:setPosition()
    local x = self.text:getLineWidthUpToIndex(self.line, self.line_index)
    self.x = self.base_x + x
    local y = self.text:getLineHeightUpToLine(self.line)
    self.y = self.base_y + y
end

function ImageCharacter:getWidth()
    return self.image:getWidth()
end

function ImageCharacter:getHeight()
    return self.image:getHeight()
end

return setmetatable({new = new}, {__call = function(_, ...) return ImageCharacter.new(...) end})
