require 'libraries/popo/utf8-l'
Character = require 'libraries/popo/Character'
ImageCharacter = require 'libraries/popo/ImageCharacter'

local Text = {}
Text.__index = Text

function Text.new(x, y, text_data) 
    local self = {}

    self.x, self.y = x, y
    self.text_data = text_data -- text_data = all text_settings + string pairs

    -- Concat the full string
    self.full_string = ''
    for i = 1, #self.text_data do
        local string = self.text_data[i]
        if type(string) == 'string' then
            self.full_string = self.full_string .. string
        end
    end

    -- Build Character objects
    self.characters = {}
    local line_index = 0
    local line = 1
    for i = 1, #self.text_data do
        -- self.text_data[i] = text settings, self.text_data[i+1] = string
        if type(self.text_data[i]) == 'table' then
            local j = i+1
            local next_text_data = self.text_data[j]
            while type(next_text_data) ~= 'table' and j <= #self.text_data do
                if type(next_text_data) == 'string' then -- string
                    local characters_as_table = {}
                    for k = 1, #self.text_data[i+1] do characters_as_table[k] = self.text_data[i+1]:utf8sub(k, k) end
                    for k = 1, #characters_as_table do
                        table.insert(self.characters, Character(self, characters_as_table[k], #self.characters, line_index, line, self.text_data[i]))
                        line_index = line_index + 1
                    end
                elseif type(next_text_data) == 'userdata' then -- Image
                    table.insert(self.characters, ImageCharacter(self, next_text_data, #self.characters, line_index, line, self.text_data[i]))
                    line_index = line_index + 1
                elseif type(next_text_data) == 'number' then -- special characters: newline, tab, etc
                    if next_text_data == 1 then
                        table.insert(self.characters, Text.NEWLINE)
                        line_index = 0
                        line = line + 1
                    end
                end
                j = j + 1
                next_text_data = self.text_data[j]
            end
        end
    end
    
    local self = setmetatable(self, Text)
    self:rebuild()
    return self
end

function Text:update(dt)
    for _, c in ipairs(self.characters) do c:update(dt) end
end

function Text:draw(ox, oy)
    for _, c in ipairs(self.characters) do c:draw(ox, oy) end
end

function Text:setFont(font)
    self.font = font
    self:rebuild()
end

function Text:setLineHeightSpacing(spacing)
    self.line_height_spacing = spacing
    self:rebuild()
end

function Text:rebuild()
    for i = 1, #self.characters do self.characters[i]:setPosition() end
end

function Text:getLineWidthUpToIndex(line, line_index)
    local characters = self:getCharactersFromLine(line)
    local width = 0
    for i = 1, #characters do
        local character = characters[i]
        if character.line_index < line_index then
            width = width + character:getWidth()
        end
    end
    return width
end

function Text:getLineHeightUpToLine(line)
    local height = 0
    for i = 1, line-1 do
        height = height + self:getLineHeight(line) + (self.line_height_spacing or 0)
    end
    return height
end

function Text:getLineHeight(line)
    local characters = self:getCharactersFromLine(line)
    local max_height = 0
    for i = 1, #characters do
        local character = characters[i]
        local height = character:getHeight()
        if height > max_height then max_height = height end
    end
    return max_height
end

function Text:getCharactersFromLine(line)
    local characters = {}
    for i = 1, #self.characters do
        if self.characters[i].line == line then table.insert(characters, self.characters[i]) end
    end
    return characters
end 

return setmetatable({new = new, NEWLINE = 1}, {__call = function(_, ...) return Text.new(...) end})
