local Timer = {}
Timer.__index = Timer

local humpTimer = require 'libraries/hump/timer'

function Timer.new()
    local self = {}

    self.timer = humpTimer.new()
    self.tags = {}
    self.pulse_tags = {}

    return setmetatable(self, Timer)
end

function Timer:update(dt)
    if self.timer then self.timer:update(dt) end
end

function Timer:tween(tag, duration, table, tween_table, tween_function, after)
    if type(tag) == 'number' or type(tag) == 'table' then
        return self.timer:tween(tag, duration, table, tween_table, tween_function, after)
    end

    if not self.tags[tag] then
        self.tags[tag] = self.timer:tween(duration, table, tween_table, tween_function, after)
    else
        self.timer:cancel(self.tags[tag])
        self.tags[tag] = self.timer:tween(duration, table, tween_table, tween_function, after)
    end
end

function Timer:after(tag, duration, func)
    if type(tag) == 'number' or type(tag) == 'table' then
        return self.timer:after(tag, duration, func)
    end

    if not self.tags[tag] then
        self.tags[tag] = self.timer:after(duration, func)
    else
        self.timer:cancel(self.tags[tag])
        self.tags[tag] = self.timer:after(duration, func)
    end
end

function Timer:every(tag, duration, func, count)
    if type(tag) == 'number' or type(tag) == 'table' then
        return self.timer:every(tag, duration, func, count)
    end

    if not self.tags[tag] then
        self.tags[tag] = self.timer:every(duration, func, count)
    else
        self.timer:cancel(self.tags[tag])
        self.tags[tag] = self.timer:every(duration, func, count)
    end
end

function Timer:during(tag, duration, func, after)
    if type(tag) == 'number' or type(tag) == 'table' then
        return self.timer:during(tag, duration, func, after)
    end

    if not self.tags[tag] then
        self.tags[tag] = self.timer:during(duration, func, after)
    else
        self.timer:cancel(self.tags[tag])
        self.tags[tag] = self.timer:during(duration, func, after)
    end
end

function Timer:pulse(tag, duration, first_half_duration, first_action, second_action)
    self.pulse_tags[tag] = true
    self:after(tag, 0, function(f)
        if type(first_half_duration) == 'number' then first_half_duration = {first_half_duration, first_half_duration} end
        local r = math.randomp(first_half_duration[1], first_half_duration[2])
        self:after(tag .. '_up', r, function()
            first_action()
            self:after(tag .. '_down', duration-r, function()
                if not second_action then first_action() else second_action() end
                self:after(tag, 0, f)
            end)
        end)
    end)
end

function Timer:pulset(tag, duration, first_half_duration, target_table, first_half_tween_table, first_half_tween_mode, second_half_tween_table, second_half_tween_mode)
    self.pulse_tags[tag] = true
    self:after(tag, 0, function(f)
        if type(first_half_duration) == 'number' then first_half_duration = {first_half_duration, first_half_duration} end
        local r = math.randomp(first_half_duration[1], first_half_duration[2])
        self:tween(tag .. '_up', r, target_table, first_half_tween_table, first_half_tween_mode, function()
            self:tween(tag .. '_down', duration-r, target_table, second_half_tween_table, second_half_tween_mode, function()
                self:after(tag, 0, f)  
            end)
        end)
    end)
end

function Timer:everyr(tag, duration, func, count, after)
    local count = count
    self:after(tag, 0, function(f)
        if (count and count == 0) then
            if after then after() end
            return
        else
            self:after(tag, math.randomp(duration[1], duration[2]), function()
                func()
                if count then count = count - 1 end
                self:after(tag, 0, f)
            end)
        end
    end)
end

function Timer:everyi(tag, duration, decrease, func, count, after)
    local decrease_count = 0
    local count = count or 100000000
    self:after(tag, 0, function(f)
        if (count and count == 0) or (duration - decrease_count*decrease <= 0) then
            if after then after() end
            return
        else
            self:after(tag, duration - decrease_count*decrease, function()
                func()
                if count then count = count - 1 end
                decrease_count = decrease_count + 1
                self:after(tag, 0, f)
            end)
        end
    end)
end


function Timer:cancel(tag)
    if self.tags[tag] then
        self.timer:cancel(self.tags[tag])
        self.tags[tag] = nil
    else self.timer:cancel(tag) end

    if self.pulse_tags[tag] then
        self.pulse_tags[tag] = nil
        self.timer:cancel(tag)
        self:cancel(tag .. '_up')
        self:cancel(tag .. '_down')
    end
end

function Timer:clear()
    self.timer:clear()
    self.tags = {}
end

function Timer:destroy()
    self.timer:clear()
    self.tags = {}
    self.timer = nil
end

return setmetatable({new = new}, {__call = function(_, ...) return Timer.new(...) end})
