local Ammo = GameObject:extend()

function Ammo:new(area, x, y, opts)
    Ammo.super.new(self, area, x, y, opts)

    self.w, self.h = 4, 4
    self:addPhysics(self.x, self.y, {collider_type = 'Polygon', vertices = createIrregularPolygon(4), collision_class = 'Collectable'})
    self.body:setPosition(self.x, self.y)
    self.body:setFixedRotation(false)
    self.r = math.randomp(0, 2*math.pi)
    self.v = math.randomp(10, 20)
    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))
    self.body:applyAngularImpulse(math.randomp(-24, 24))
end

function Ammo:update(dt)
    Ammo.super.update(self, dt)

    local target = self.area.stage.player
    if target then
        local projectile_heading = Vector(self.body:getLinearVelocity()):normalized()
        local angle = math.atan2(target.y - self.y, target.x - self.x)
        local to_target_heading = Vector(math.cos(angle), math.sin(angle)):normalized()
        local final_heading = (projectile_heading + 0.1*to_target_heading):normalized()
        self.body:setLinearVelocity(self.v*final_heading.x, self.v*final_heading.y)
    else self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r)) end
end

function Ammo:draw()
    love.graphics.setColor(ammo_color)
    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    for i = 1, #points, 2 do
        if i < #points-2 then 
            love.graphics.line(points[i], points[i+1], points[i+2], points[i+3]) 
        elseif i >= #points-2 then
            love.graphics.line(points[i], points[i+1], points[1], points[2]) 
        end
    end
    love.graphics.setColor(255, 255, 255)
end

function Ammo:destroy()
    Ammo.super.destroy(self)
end

function Ammo:die()
    self.dead = true

    for i = 1, love.math.random(4, 8) do self.area:addGameObject('ExplodeParticle', self.x, self.y, {r = 3, color = ammo_color}) end

    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    local cx, cy = Math.polygon.getCentroid(points)
    for i = 1, #points, 2 do
        if i < #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[i+2], points[i+3]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = ammo_color})
        elseif i >= #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[1], points[2]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = ammo_color})
        end
    end
end

return Ammo
