local Attack = GameObject:extend()

function Attack:new(area, x, y, opts)
    Attack.super.new(self, area, x, y, opts)

    self.w, self.h = 10, 10
    self.sx, self.sy = 1, 1
    self.x = table.random({0 - 40, gw + 40})
    self.y = math.randomp(40, gh - 40)
    if opts.x and opts.y then self.x, self.y = opts.x, opts.y end
    self.vertices = createIrregularPolygon(self.w, 8)
    self:addPhysics(self.x, self.y, {collider_type = 'Circle', r = self.w, collision_class = 'Collectable'})
    self.body:setPosition(self.x, self.y)
    self.body:setFixedRotation(false)
    self.v = opts.v or math.randomp(20, 40)
    self.r = opts.r or 0
    if not opts.r and math.sign(self.x) > 0 then self.r = math.pi end
    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))
    self.body:applyAngularImpulse(math.randomp(-100, 100))

    self.attack = attacks[love.math.random(1, #attacks)]
end

function Attack:update(dt)
    Attack.super.update(self, dt)

    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r)) 
    if self.y < -40 then self:hit() end
    if self.y > gh + 40 then self:hit() end
end

function Attack:draw()
    love.graphics.setColor(default_color)
    love.graphics.print(attack_abbreviations[self.attack], self.x, self.y - 2, 0, 1, 1, math.floor(font:getWidth(attack_abbreviations[self.attack])/2), math.floor(font:getHeight()/2))

    love.graphics.circle('line', self.x, self.y, self.w)
    love.graphics.setColor(255, 255, 255)
end

function Attack:destroy()
    Attack.super.destroy(self)
end

function Attack:die()
    self.dead = true

    for i = 1, love.math.random(4, 8) do self.area:addGameObject('ExplodeParticle', self.x, self.y, {r = 3, color = hp_color}) end

    local points = fn.map(self.vertices, function(k, v) if k % 2 == 1 then return self.x + v else return self.y + v end end)
    local cx, cy = Math.polygon.getCentroid(points)
    for i = 1, #points, 2 do
        if i < #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[i+2], points[i+3]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = hp_color})
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = default_color})
        elseif i >= #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[1], points[2]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = hp_color})
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = default_color})
        end
    end
end

return Attack
