local BigRock = GameObject:extend()

function BigRock:new(area, x, y, opts)
    BigRock.super.new(self, area, x, y, opts)

    self.w, self.h = 16, 16
    self.x = table.random({0 - 40, gw + 40})
    self.y = math.randomp(40, gh - 40)
    if opts.x and opts.y then self.x, self.y = opts.x, opts.y end
    self.d = -math.sign(self.x)
    self:addPhysics(self.x, self.y, {collider_type = 'Polygon', vertices = createIrregularPolygon(16), collision_class = 'Enemy'})
    self.body:setPosition(self.x, self.y)
    self.body:setFixedRotation(false)
    self.v = opts.v or math.randomp(20, 40)
    self.r = opts.r or 0
    if math.sign(self.x) > 0 then self.r = math.pi end
    self.body:applyAngularImpulse(math.randomp(-100, 100))
end

function BigRock:update(dt)
    BigRock.super.update(self, dt)

    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))
    if self.y < -40 then self:hit() end
    if self.y > gh + 40 then self:hit() end
end

function BigRock:draw()
    love.graphics.setColor(255, 32, 32)
    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    for i = 1, #points, 2 do
        if i < #points-2 then 
            love.graphics.line(points[i], points[i+1], points[i+2], points[i+3]) 
        elseif i >= #points-2 then
            love.graphics.line(points[i], points[i+1], points[1], points[2]) 
        end
    end
    love.graphics.setColor(255, 255, 255)
end

function BigRock:destroy()
    BigRock.super.destroy(self)
end

function BigRock:hit()
    if self.dead then return end
    self.dead = true

    self.area:addGameObject('Ammo', self.x, self.y)
    for i = 1, 4 do self.area:addGameObject('Rock', 0, 0, {x = self.x, y = self.y, r = self.r}) end
    score = score + 200

    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    local cx, cy = Math.polygon.getCentroid(points)
    for i = 1, #points, 2 do
        if i < #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[i+2], points[i+3]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = {255, 32, 32}})
        elseif i >= #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[1], points[2]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = {255, 32, 32}})
        end
    end
end

return BigRock
