local Chain = GameObject:extend()

function Chain:new(area, x, y, opts)
    Chain.super.new(self, area, x, y, opts)

    self.chain_part_w, self.chain_part_h = opts.chain_part_w or 6, opts.chain_part_h or 3
    self.chain_parts_amount = opts.chain_parts_amount or love.math.random(4, 6)

    self.chain_parts = {}
    self.joints = {}
    table.insert(self.chain_parts, self.area:addGameObject('ChainPart', self.x, self.y, {parent = self, w = self.chain_part_w, h = self.chain_part_h, color = self.color}))
    for i = 1, self.chain_parts_amount-1 do
        local last_chain_part = self.chain_parts[#self.chain_parts]
        local chain_part = self.area:addGameObject('ChainPart', self.x + i*self.chain_part_w, self.y, {parent = self, w = self.chain_part_w, h = self.chain_part_h, color = self.color})
        local joint = self.area.world:addJoint('RevoluteJoint', last_chain_part.body, chain_part.body, (last_chain_part.x + chain_part.x)/2, (last_chain_part.y + chain_part.y)/2, true)
        table.insert(self.chain_parts, chain_part)
        table.insert(self.joints, joint)
    end

    for i, joint in ipairs(self.joints) do 
        joint:setLimitsEnabled(true) 
        joint:setLimits(-math.pi/2, math.pi/2)
    end

    local middle_chain_part = self.chain_parts[math.ceil(self.chain_parts_amount/2)]
    middle_chain_part.body:applyAngularImpulse(-self.d*math.randomp(1500, 2500))
end

function Chain:update(dt)
    Chain.super.update(self, dt)
end

function Chain:draw()

end

function Chain:destroy()
    Chain.super.destroy(self)
end

function Chain:attachFirst(object, x, y)
    local first_chain_part = self.chain_parts[1]
    table.insert(self.joints, 1, self.area.world:addJoint('RevoluteJoint', object.body, first_chain_part.body, x, y, true))

    for i = 2, #self.joints do
        local joint = self.joints[i]
        joint:setLimitsEnabled(true) 
        joint:setLimits(-math.pi/2, math.pi/2)
    end
end

function Chain:unlink()
    for i, joint in ipairs(self.joints) do 
        self.timer:after((i-1)*0.1, function() joint:destroy() end)
    end
end

return Chain
