local ChainPart = GameObject:extend()

function ChainPart:new(area, x, y, opts)
    ChainPart.super.new(self, area, x, y, opts)
    self:addPhysics(x, y, {w = self.w, h = self.h, collision_class = 'Enemy'})
    self.body:setGravityScale(1)
    self.body:setFixedRotation(false)
    self.collider.fixtures['main']:setFriction(1)
end

function ChainPart:update(dt)
    ChainPart.super.update(self, dt)
end

function ChainPart:draw()
    love.graphics.setColor(self.color)
    pushRotate(self.x, self.y, self.body:getAngle())
    love.graphics.rectangle('line', self.x - self.w/2, self.y - self.h/2, self.w, self.h)
    love.graphics.pop()
    love.graphics.setColor(255, 255, 255)
end

function ChainPart:destroy()
    ChainPart.super.destroy(self)
end

function ChainPart:hit()
    if self.dead then return end

    self.dead = true
    
    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    local cx, cy = Math.polygon.getCentroid(points)
    for i = 1, #points, 2 do
        if i < #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[i+2], points[i+3]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = self.color})
        elseif i >= #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[1], points[2]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = self.color})
        end
    end
end

return ChainPart
