local EnemyProjectile = GameObject:extend()

function EnemyProjectile:new(area, x, y, opts)
    EnemyProjectile.super.new(self, area, x, y, opts)

    self.s = opts.s or 2.5
    self:addPhysics(x, y, {collider_type = 'Circle', r = self.s, collision_class = 'EnemyProjectile'})
    self.v = opts.v or 200
    self.r = opts.r or math.randomp(0, 2*math.pi)
    self.rv = opts.rv or table.random({math.randomp(-2*math.pi, -math.pi), math.randomp(math.pi, 2*math.pi)})
    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))

    self.timer:every(0.01, function() 
        self.area:addGameObject('TrailParticle', self.x + math.randomp(-1, 1), self.y + math.randomp(-1, 1), 
        {r = math.randomp(self.s - 1, self.s), d = opts.d or math.randomp(0.08, 0.12), color = enemy_color}) 
    end)

    if self.spin then self.timer:after(math.randomp(2.4, 3.2), function() self:die() end) end
    if self.mine then 
        self.rv = opts.rv or table.random({math.randomp(-12*math.pi, -10*math.pi), math.randomp(10*math.pi, 12*math.pi)})
        self.timer:tween(math.randomp(12, 14), self, {rv = math.randomp(self.rv/12, self.rv/10)}, 'linear', function() self:die() end) 
    end
    if self.shield then
        self.orbitting = true
        self.orbit_distance = math.randomp(32, 48)
        self.orbit_offset = math.randomp(0, 2*math.pi)
        self.orbit_speed = math.randomp(-6, 6)
    end
end

function EnemyProjectile:update(dt)
    EnemyProjectile.super.update(self, dt)

    if self.x < -160 then self:die() end
    if self.y < -160 then self:die() end
    if self.x > gw + 160 then self:die() end
    if self.y > gh + 160 then self:die() end

    if self.collider:enter('Player') then
        local _, collider = self.collider:enter('Player')
        local other = collider.user_data
        if other then
            other:hit()
            self:die()
        end
    end

    if self.collider:enter('Projectile') then
        local _, collider = self.collider:enter('Projectile')
        local other = collider.user_data
        if other then
            other:die()
            self:die()
        end
    end

    if self.spin or self.mine then self.r = self.r + self.rv*dt end
    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))

    if self.shield and self.orbitting then
        self.body:setPosition(self.parent.x + self.orbit_distance*math.cos(self.orbit_speed*time + self.orbit_offset), self.parent.y + self.orbit_distance*math.sin(self.orbit_speed*time + self.orbit_offset))
    end
end

function EnemyProjectile:draw()

end

function EnemyProjectile:destroy()
    EnemyProjectile.super.destroy(self)
end

function EnemyProjectile:die()
    if self.dead then return end
    self.dead = true
    for i = 1, love.math.random(4, 8) do self.area:addGameObject('ExplodeParticle', self.x, self.y, {r = self.s, color = enemy_color}) end
end

function EnemyProjectile:setSpin()
    if self.dead then return end
    self.spin = true
    self.timer:after(math.randomp(2.4, 4.8), function() self:die() end)
end

return EnemyProjectile
