local Explosion = GameObject:extend()

function Explosion:new(area, x, y, opts)
    Explosion.super.new(self, area, x, y, opts)

    self.r = 16
    self.color = opts.color or {255, 255, 255}
    self.timer:tween(0.1, self, {r = opts.r or math.randomp(48, 56)}, 'in-out-cubic', function()
        camera:shake(2, 60, 0.5)
        for i = 1, love.math.random(8, 12) do self.area:addGameObject('ExplodeParticle', self.x, self.y, {v = math.random(150, 300), r = math.randomp(6, 10), color = self.color}) end
        self.timer:tween(0.20, self, {r = 0}, 'in-out-cubic', function() self.dead = true end)
    end)

    local objects = self.area:queryCircleArea(self.x, self.y, 48, enemies)
    table.sort(objects, function(a, b) 
        local da = distance(self.x, self.y, a.x, a.y)
        local db = distance(self.x, self.y, b.x, b.y)
        return da < db
    end)
    for i, object in ipairs(objects) do self.timer:after((i-1)*0.025, function() object:hit() end) end
end

function Explosion:update(dt)
    Explosion.super.update(self, dt)
end

function Explosion:draw()
    love.graphics.setColor(self.color)
    love.graphics.circle('fill', self.x, self.y, self.r)
    love.graphics.setColor(255, 255, 255)
end

function Explosion:destroy()
    Explosion.super.destroy(self)
end

return Explosion
