local Orbitter = GameObject:extend()

function Orbitter:new(area, x, y, opts)
    Orbitter.super.new(self, area, x, y, opts)

    self.w, self.h = 12, 12
    self.x = table.random({0 - 80, gw + 80})
    self.y = math.randomp(40, gh - 40)
    if opts.x and opts.y then self.x, self.y = opts.x, opts.y end
    self:addPhysics(self.x, self.y, {collider_type = 'Polygon', vertices = createIrregularPolygon(12), collision_class = 'Enemy'})
    self.body:setPosition(self.x, self.y)
    self.body:setFixedRotation(false)
    self.v = opts.v or math.randomp(20, 40)
    self.r = opts.r or 0
    if math.sign(self.x) > 0 then self.r = math.pi end
    self.body:applyAngularImpulse(math.randomp(-100, 100))

    self.projectiles = {}
    self.timer:after(0.5, function()
        for i = 1, 5 do table.insert(self.projectiles, self.area:addGameObject('EnemyProjectile', self.x - math.sign(self.x)*40, self.y, {parent = self, shield = true, r = self.body:getAngle()})) end
    end)
end

function Orbitter:update(dt)
    Orbitter.super.update(self, dt)

    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))
    if self.y < -40 then self:hit() end
    if self.y > gh + 40 then self:hit() end
end

function Orbitter:draw()
    love.graphics.setColor(enemy_color)
    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    for i = 1, #points, 2 do
        if i < #points-2 then 
            love.graphics.line(points[i], points[i+1], points[i+2], points[i+3]) 
        elseif i >= #points-2 then
            love.graphics.line(points[i], points[i+1], points[1], points[2]) 
        end
    end
    love.graphics.setColor(255, 255, 255)
end

function Orbitter:destroy()
    Orbitter.super.destroy(self)

    if self.y < -40 then self:hit() end
    if self.y > gh + 40 then self:hit() end
end

function Orbitter:hit()
    if self.dead then return end
    self.dead = true

    for _, projectile in ipairs(self.projectiles) do 
        projectile.shield = false 
        projectile:setSpin()
    end

    self.area:addGameObject('Ammo', self.x, self.y)
    score = score + 400

    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    local cx, cy = Math.polygon.getCentroid(points)
    for i = 1, #points, 2 do
        if i < #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[i+2], points[i+3]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = enemy_color})
        elseif i >= #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[1], points[2]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = enemy_color})
        end
    end
end

return Orbitter
