local Player = GameObject:extend()

function Player:new(area, x, y, opts)
    Player.super.new(self, area, x, y, opts)

    self.build = PlayerBuild(self)

    self.x, self.y = x, y
    self.w, self.h = self.build.size_multiplier*8, self.build.size_multiplier*8
    self:addPhysics(x, y, {collider_type = 'Circle', r = self.w, collision_class = 'Player'})
    self.vertices = {
        self.w, 0,
        self.w - self.w/2, self.w/2,
        -self.w, self.w/2,
        -self.w, -self.w/2,
        self.w - self.w/2, -self.w/2,
    }

    self.r = -math.pi/2
    self.rv = 1.66*math.pi
    self.v = 0
    self.init_max_v = 100
    self.max_v = 100
    self.a = 100

    self.buffs = {}
    self.buff_size_multiplier = 1

    -- Trail
    self.color = {255, 255, 255}
    self.timer:every(0.01, function() 
        self.area:addGameObject('TrailParticle', 
        self.x - 1.0*self.buff_size_multiplier*self.w*math.cos(self.r) + 0.2*self.buff_size_multiplier*self.w*math.cos(self.r - math.pi/2), 
        self.y - 1.0*self.buff_size_multiplier*self.w*math.sin(self.r) + 0.2*self.buff_size_multiplier*self.w*math.sin(self.r - math.pi/2), 
        {parent = self, r = math.randomp(2, 4)*self.build.size_multiplier*self.buff_size_multiplier, d = math.randomp(0.15, 0.25), color = self.color}) 
        self.area:addGameObject('TrailParticle', 
        self.x - 1.0*self.buff_size_multiplier*self.w*math.cos(self.r) + 0.2*self.buff_size_multiplier*self.w*math.cos(self.r + math.pi/2), 
        self.y - 1.0*self.buff_size_multiplier*self.w*math.sin(self.r) + 0.2*self.buff_size_multiplier*self.w*math.sin(self.r + math.pi/2), 
        {parent = self, r = math.randomp(2, 4)*self.build.size_multiplier*self.buff_size_multiplier, d = math.randomp(0.15, 0.25), color = self.color}) 
    end)

    -- Boost
    self.timer:every(0.1, function() boost = math.min(boost + 1*self.build.boost_recharge_multiplier.value, max_boost) end)
    self.can_boost = true
    self.boost_timer = 0
    self.boost_cooldown = 2

    -- Shoot
    self.shoot_cooldown = 0
    self.shoot_cooldown_timer = 0

    self:setAttack('Neutral')
end

function Player:update(dt)
    Player.super.update(self, dt)
    self.build:update(dt)

    -- Movement
    self.max_v = self.init_max_v
    local d = input:down('horizontal') or 0
    
    if d < -0.25 or input:down('left') then self.r = self.r - self.build.turn_rate_multiplier.value*self.rv*dt end
    if d > 0.25 or input:down('right') then self.r = self.r + self.build.turn_rate_multiplier.value*self.rv*dt end

    -- Boost
    self.boost_timer = self.boost_timer + dt
    self.boosting = false
    if self.boost_timer > self.boost_cooldown then self.can_boost = true end
    if input:pressed('up') and boost > 1 and self.can_boost then self.build:onBoostStart() end
    if input:released('up') then self.build:onBoostEnd() end
    if input:down('up') and boost > 1 and self.can_boost then 
        self.max_v = 1.5*self.init_max_v*self.build.boost_effectiveness_multiplier.value
        boost = boost - 50*dt
        if boost <= 1 then
            self.can_boost = false
            self.boost_timer = 0
            self.build:onBoostEnd()
        end
    end
    if input:pressed('down') and boost > 1 and self.can_boost then self.build:onBoostStart() end
    if input:released('down') then self.build:onBoostEnd() end
    if input:down('down') and boost > 1 and self.can_boost then 
        self.max_v = 0.5*self.init_max_v*(2-self.build.boost_effectiveness_multiplier.value)
        boost = boost - 50*dt
        if boost <= 1 then
            self.can_boost = false
            self.boost_timer = 0
            self.build:onBoostEnd()
        end
    end

    self.v = math.min(self.v + self.a*dt, self.max_v*self.build.mvspd_multiplier.value)
    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))

    -- Shoot
    self.shoot_cooldown_timer = self.shoot_cooldown_timer + dt
    if self.shoot_cooldown_timer > self.shoot_cooldown*self.build.aspd_multiplier.value then
        self.shoot_cooldown_timer = 0
        self:shoot()
    end

    -- Collision
    if self.x < 0 then self:hit(10) end
    if self.y < 0 then self:hit(10) end
    if self.x > gw then self:hit(10) end
    if self.y > gh then self:hit(10) end

    if self.collider:enter('Enemy') then
        local _, collider = self.collider:enter('Enemy')
        local other = collider.user_data
        if other then self:hit() end
    end

    if self.collider:enter('Collectable') then
        local _, collider = self.collider:enter('Collectable')
        local other = collider.user_data

        if other then
            if other.class_name == 'Ammo' then
                other:die()
                self:addAmmo(5)
                self.build:onAmmoPickup(other)

            elseif other.class_name == 'Boost' then
                other:die()
                self:addBoost(20)
                self.build:onBoostPickup(other)

            elseif other.class_name == 'HP' then
                other:die()
                self:addHP(1)
                self.build:onHPPickup(other)

            elseif other.class_name == 'SkillPoint' then
                other:die()
                self:addSkillPoint(1)
                self.build:onSPPickup(other)

            elseif other.class_name == 'Attack' then
                other:die()
                self:setAttack(other.attack)
            end
        end
    end
end

function Player:draw()
    if self.disappearing then return end

    pushRotateScale(self.x, self.y, self.r, self.buff_size_multiplier, self.buff_size_multiplier)
    love.graphics.setColor(self.color)
    local points = fn.map(self.vertices, function(k, v) if k % 2 == 1 then return self.x + v + math.randomp(-1, 1) else return self.y + v + math.randomp(-1, 1) end end)
    love.graphics.polygon('line', points)
    love.graphics.pop()
end

function Player:destroy()
    Player.super.destroy(self)
end

function Player:shoot()
    local d
    if self.build.size_multiplier > 1 then d = 2.2*self.w/(0.8*self.build.size_multiplier)
    else d = 2.2*self.w end
    local mods = {}
    mods.parent = self
    mods.color = table.copy(self.color)
    mods.random_degree = self.build.random_degree 
    mods.ninety_degree = self.build.ninety_degree
    mods.fast_slow = self.build.fast_slow
    mods.slow_fast = self.build.slow_fast
    mods.wavy = self.build.wavy
    if self.build.rolls.shield_projectile_chance:next() then 
        mods.shield = true 
        mods.s = 4.5 + self.build.additional_projectile_size
    end

    if self.attack == 'Neutral' then
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r}, mods))

    elseif self.attack == 'Double' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r + math.pi/12), self.y + d*math.sin(self.r + math.pi/12), table.merge({r = self.r + math.pi/12}, mods))
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r - math.pi/12), self.y + d*math.sin(self.r - math.pi/12), table.merge({r = self.r - math.pi/12}, mods))

    elseif self.attack == 'Triple' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r}, mods))
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r + math.pi/12), self.y + d*math.sin(self.r + math.pi/12), table.merge({r = self.r + math.pi/12}, mods))
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r - math.pi/12), self.y + d*math.sin(self.r - math.pi/12), table.merge({r = self.r - math.pi/12}, mods))

    elseif self.attack == 'Rapid' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r}, mods))

    elseif self.attack == 'Spread' then
        ammo = ammo - attack_ammo[self.attack]
        local random_angle = math.randomp(-math.pi/8, math.pi/8)
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r + random_angle), self.y + d*math.sin(self.r + random_angle), table.merge({r = self.r + random_angle}, mods))

    elseif self.attack == 'Spin' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r, spin = true}, mods))

    elseif self.attack == 'Split' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r, split = true}, mods))

    elseif self.attack == 'Homing' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r, homing = true}, mods))

    elseif self.attack == 'Blast' then
        ammo = ammo - attack_ammo[self.attack]
        for i = 1, 12 do
            local random_angle = math.randomp(-math.pi/6, math.pi/6)
            self.area:addGameObject('Projectile', self.x + d*math.cos(self.r + random_angle), self.y + d*math.sin(self.r + random_angle), 
            table.merge({r = self.r + random_angle, v = math.randomp(500, 600), blast = true, s = 2.5}, mods))
        end

    elseif self.attack == 'Back' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r}, mods))
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r - math.pi), self.y + d*math.sin(self.r - math.pi), table.merge({r = self.r - math.pi}, mods))

    elseif self.attack == 'Side' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r}, mods))
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r - math.pi/2), self.y + d*math.sin(self.r - math.pi/2), table.merge({r = self.r - math.pi/2}, mods))
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r + math.pi/2), self.y + d*math.sin(self.r + math.pi/2), table.merge({r = self.r + math.pi/2}, mods))

    elseif self.attack == 'Laser' then
        ammo = ammo - attack_ammo[self.attack]
        local x1 = self.x + d*math.cos(self.r)
        local y1 = self.y + d*math.sin(self.r)
        local x2 = self.x + 96*math.cos(self.r)
        local y2 = self.y + 96*math.sin(self.r)
        for i = 1, love.math.random(4, 8) do 
            self.area:addGameObject('ExplodeParticle', self.x + 12*math.cos(self.r), self.y + 12*math.sin(self.r), {v = math.randomp(25, 50), r = math.randomp(1.5, 3), d = math.randomp(0.2, 0.4)}) 
        end
        self.area:addGameObject('LightningLine', 0, 0, {x1 = x1, y1 = y1, x2 = x2, y2 = y2})
        local objects_1 = self.area:queryLine(x1, y1, x2, y2, enemies)
        local objects_2 = self.area:queryLine(x1 + 10*math.cos(self.r - math.pi/2), y1 + 10*math.sin(self.r - math.pi/2), x2 + 10*math.cos(self.r - math.pi/2), y2 + 10*math.sin(self.r - math.pi/2), enemies)
        local objects_3 = self.area:queryLine(x1 + 10*math.cos(self.r + math.pi/2), y1 + 10*math.sin(self.r + math.pi/2), x2 + 10*math.cos(self.r + math.pi/2), y2 + 10*math.sin(self.r + math.pi/2), enemies)
        local objects = table.merge(table.merge(objects_1, objects_2), objects_3)
        for _, object in ipairs(objects) do object:hit() end

    elseif self.attack == 'Flame' then
        ammo = ammo - attack_ammo[self.attack]
        local random_angle = math.randomp(-math.pi/d, math.pi/d)
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r + random_angle), self.y + d*math.sin(self.r + random_angle), 
        table.merge({flame = true, s = 2, r = self.r + random_angle, d = math.randomp(0.12, 0.24)}, mods))

    elseif self.attack == 'Big' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r, s = 8, big = true}, mods))

    elseif self.attack == 'Explode' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r, explode = true}, mods))

    elseif self.attack == 'Lightning' then
        local x1 = self.x + d*math.cos(self.r)
        local y1 = self.y + d*math.sin(self.r)
        local closest = self.area:getClosestGameObject(self.id, x1 + 24*math.cos(self.r), y1 + 24*math.sin(self.r), 48, enemies)
        if closest then
            ammo = ammo - attack_ammo[self.attack]
            closest:hit()
            local x2, y2 = closest.x, closest.y
            self.area:addGameObject('LightningLine', 0, 0, {x1 = x1, y1 = y1, x2 = x2, y2 = y2})
            for i = 1, love.math.random(4, 8) do self.area:addGameObject('ExplodeParticle', x1, y1) end
            for i = 1, love.math.random(4, 8) do self.area:addGameObject('ExplodeParticle', x2, y2) end
        end

    elseif self.attack == 'Bounce' then
        ammo = ammo - attack_ammo[self.attack]
        self.area:addGameObject('Projectile', self.x + d*math.cos(self.r), self.y + d*math.sin(self.r), table.merge({r = self.r, bounce = 4}, mods))
    end

    if ammo <= 0 then 
        self:setAttack('Neutral')
        ammo = max_ammo
    end

    if self.build.rolls.double_attack_chance:next() then
        self:addBuff(self.shoot_cooldown/2, attack_color)
        self.timer:after(self.shoot_cooldown/2, function() self:shoot() end)
    end
end

function Player:hit(damage)
    if self.invincible then return end
    self.invincible = true
    self.timer:after('invincibility', 2*self.build.invulnerability_multiplier.value, function() self.invincible = false end)

    camera:shake(6, 60, 0.2)
    flash(3)
    slow(0.25/self.build.invulnerability_multiplier.value, 0.5)
    for i = 1, love.math.random(4, 8) do self.area:addGameObject('ExplodeParticle', self.x, self.y) end

    for i = 1, math.floor(50*self.build.invulnerability_multiplier.value) do self.timer:after((i-1)*0.04, function() self.disappearing = not self.disappearing end) end
    self.timer:after((math.floor(50*self.build.invulnerability_multiplier.value)+1)*0.04, function() self.disappearing = false end)

    self:removeHP(damage)
end

function Player:die()
    if self.dead then return end
    self.dead = true 
    for i = 1, love.math.random(8, 12) do self.area:addGameObject('ExplodeParticle', self.x, self.y) end
    current_room:finish()

    if self.build.gain_sp_on_death then 
        self:addSkillPoint(20) 
        self.area:addGameObject('IrregularPolygonEffect', self.x, self.y, {parent = self, w = 4, color = skill_point_color})
    end
end

function Player:setAttack(attack)
    self.attack = attack
    self.shoot_cooldown = attack_cooldowns[attack]
    ammo = max_ammo
end

function Player:setRandomAttack()
    self.attack = attacks[love.math.random(1, #attacks)]
    self.shoot_cooldown = attack_cooldowns[self.attack]
end

function Player:addHP(amount)
    hp = math.min(hp + amount, max_hp)
end

function Player:removeHP(amount)
    hp = hp - (amount or 1)
    if hp <= 0 then
        hp = 0
        self:die()
    end
end

function Player:addAmmo(amount)
    ammo = math.min(ammo + amount + ammo_gain, max_ammo)
    score = score + 50
end

function Player:addBoost(amount)
    boost = math.min(boost + amount + boost_gain, max_boost)
    score = score + 150
end

function Player:addSkillPoint(amount)
    skill_point = skill_point + amount
    score = score + 250

    if self.build.rolls.double_sp_gain_chance:next() then
        skill_point = skill_point + amount
        self.area:addGameObject('IrregularPolygonEffect', self.x, self.y, {parent = self, w = 4, color = skill_point_color})
    end
end

function Player:addBuff(duration, color)
    local id = Lume.uuid()
    table.insert(self.buffs, {id = id, color = color})
    self:updateBuffs()
    if duration then
        self.timer:after(duration, function()
            for i, buff in ipairs(self.buffs) do
                if buff.id == id then self:removeBuff(buff.id) end
            end
        end)
    end
    return id
end

function Player:removeBuff(id)
    for i, buff in ipairs(self.buffs) do
        if buff.id == id then table.remove(self.buffs, i); break end
    end
    self:updateBuffs()
end

function Player:updateBuffs()
    self.buff_size_multiplier = 1.5*(1 + (0.4*#self.buffs))
    self.timer:tween('buff_size_multiplier', 0.25, self, {buff_size_multiplier = 1 + (0.4*#self.buffs)}, 'in-out-cubic')

    local final_color
    if #self.buffs == 0 then final_color = default_color 
    else final_color = self.buffs[#self.buffs].color end
    self.timer:tween('buff_color', 0.25, self, {color = final_color}, 'in-out-cubic')
end

return Player
