local PlayerBuild = Object:extend()

function PlayerBuild:new(player)
    self.player = player
    self.timer = Timer()

    -- Projectile mods
    self.random_degree = false
    self.ninety_degree = false
    self.fast_slow = false
    self.slow_fast = false
    self.wavy = false 

    self.gain_sp_on_death = false
    self.increased_luck_while_boosting = false
    self.invulnerability_while_boosting = false
    self.launch_homing_projectile_while_boosting = false
    self.convert_hp_to_sp_if_hp_full = false

    self.additional_projectile_size = 0
    self.aspd_multiplier = Stat(1)
    self.angle_change_frequency_multiplier = Stat(1)
    self.boost_effectiveness_multiplier = Stat(1)
    self.boost_recharge_multiplier = Stat(1)
    self.invulnerability_multiplier = Stat(1)
    self.luck_multiplier = 1
    self.mvspd_multiplier = Stat(1)
    self.projectile_acceleration_multiplier = Stat(1)
    self.projectile_deceleration_multiplier = Stat(1)
    self.projectile_waviness_multiplier = Stat(1)
    self.pspd_multiplier = Stat(1)
    self.size_multiplier = 1
    self.turn_rate_multiplier = Stat(1)

    -- Chances
    self.aspd_boost_on_kill_chance = 0
    self.barrage_on_kill_chance = 0
    self.change_attack_on_kill_chance = 0
    self.double_ammo_drop_chance = 0
    self.double_attack_chance = 0
    self.double_sp_gain_chance = 0
    self.launch_homing_projectile_on_ammo_pickup_chance = 0
    self.launch_homing_projectile_on_kill_chance = 0
    self.launch_homing_projectile_while_boosting_chance = 0
    self.mine_drop_chance = 0
    self.mvspd_boost_on_kill_chance = 0
    self.additional_on_kill_event_chance = 0
    self.pspd_boost_on_kill_chance = 0
    self.pspd_inhibit_on_kill_chance = 0
    self.regain_ammo_on_kill_chance = 0
    self.regain_boost_on_kill_chance = 0
    self.regain_full_ammo_on_kill_chance = 0
    self.regain_hp_on_ammo_pickup_chance = 0
    self.regain_hp_on_kill_chance = 0
    self.regain_hp_on_sp_pickup_chance = 0
    self.shield_projectile_chance = 0
    self.spawn_fast_attacking_area_on_ammo_pickup_chance = 0
    self.spawn_fast_attacking_area_on_hp_pickup_chance = 0
    self.spawn_fast_attacking_area_on_sp_pickup_chance = 0
    self.spawn_boost_on_kill_chance = 0
    self.spawn_hp_on_kill_chance = 0
    self.spawn_sp_on_kill_chance = 0
    self:generateChances()

    -- Mine Drop
    if self.mine_drop_chance > 0 then
        self.timer:every(0.5, function()
            if self.rolls.mine_drop_chance:next() then
                local d = 2.2*self.player.w
                self.player.area:addGameObject('Projectile', self.player.x - d*math.cos(self.player.r), self.player.y - d*math.sin(self.player.r), 
                {r = self.player.r, rv = table.random({-12*math.pi, -10*math.pi}, {10*math.pi, 12*math.pi}), mine = true, parent = self.player, color = self.player.color})
            end
        end)
    end
end

function PlayerBuild:update(dt)
    self.timer:update(dt)

    if self.aspd_boosting then self.aspd_multiplier:decrease(100) end
    if self.mvspd_boosting then self.mvspd_multiplier:increase(50) end
    if self.pspd_boosting then self.pspd_multiplier:increase(100) end
    if self.pspd_inhibiting then self.pspd_multiplier:decrease(50) end

    for k, stat in pairs(self) do if type(stat) == 'table' and stat.class_name == 'Stat' then stat:update(dt) end end
end

function PlayerBuild:draw()

end

function PlayerBuild:generateChances()
    self.rolls = {}
    for k, v in pairs(self) do
        if k:find('chance') and type(v) == 'number' then
            if k:find('on_kill') and v > 0 then
                self.rolls[k] = chanceList({true, math.ceil((v+self.additional_on_kill_event_chance)*self.luck_multiplier)}, 
                {false, 100-math.ceil((v+self.additional_on_kill_event_chance)*self.luck_multiplier)})
            else
                self.rolls[k] = chanceList({true, math.ceil(v*self.luck_multiplier)}, {false, 100-math.ceil(v*self.luck_multiplier)})
            end
        end
    end
end

function PlayerBuild:onKill(enemy)
    if self.rolls.aspd_boost_on_kill_chance:next() then 
        self.aspd_boosting = true
        self.timer:after('aspd_boosting', 4, function() self.aspd_boosting = false end) 
        self.player:addBuff(4, aspd_color)
    end

    if self.rolls.barrage_on_kill_chance:next() then
        for i = 1, 8 do
            self.timer:after((i-1)*0.1, function()
                local random_angle = math.randomp(-math.pi/8, math.pi/8)
                local d = 2.2*self.player.w
                self.player.area:addGameObject('Projectile', self.player.x + d*math.cos(self.player.r + random_angle), self.player.y + d*math.sin(self.player.r + random_angle), 
                {r = self.player.r + random_angle, color = barrage_color, parent = self.player})
            end)
        end
        self.player:addBuff(0.8, barrage_color)
    end

    if self.rolls.change_attack_on_kill_chance:next() then
        self.player:setRandomAttack()
    end

    if self.rolls.double_ammo_drop_chance:next() then
        local ammo = self.player.area:addGameObject('Ammo', enemy.x, enemy.y)
        self.player.area:addGameObject('IrregularPolygonEffect', enemy.x, enemy.y, {parent = ammo, w = ammo.w, color = ammo_color})
    end
    
    if self.rolls.launch_homing_projectile_on_kill_chance:next() then
        local d = 2.2*self.player.w
        self.player.area:addGameObject('Projectile', self.player.x + d*math.cos(self.player.r), self.player.y + d*math.sin(self.player.r), 
        {r = self.player.r, homing = true, parent = self.player, color = barrage_color})
        self.player:addBuff(0.2, barrage_color)
    end

    if self.rolls.mvspd_boost_on_kill_chance:next() then
        self.mvspd_boosting = true
        self.timer:after('mvspd_boosting', 4, function() self.mvspd_boosting = false end) 
        self.player:addBuff(4, mvspd_color)
    end

    if self.rolls.pspd_boost_on_kill_chance:next() then
        self.pspd_boosting = true
        self.timer:after('pspd_boosting', 4, function() self.pspd_boosting = false end) 
        self.player:addBuff(4, pspd_color)
    end

    if self.rolls.pspd_inhibit_on_kill_chance:next() then
        self.pspd_inhibiting = true
        self.timer:after('pspd_inhibiting', 4, function() self.pspd_inhibiting = false end) 
        self.player:addBuff(4, pspd_color)
    end

    if self.rolls.regain_ammo_on_kill_chance:next() then
        self.player:addAmmo(20)
        self.player.area:addGameObject('IrregularPolygonEffect', self.player.x, self.player.y, {parent = self.player, w = 4, color = ammo_color})
    end

    if self.rolls.regain_boost_on_kill_chance:next() then
        self.player:addBoost(20)
        self.player.area:addGameObject('IrregularPolygonEffect', self.player.x, self.player.y, {parent = self.player, w = 4, color = boost_color})
    end

    if self.rolls.regain_full_ammo_on_kill_chance:next() then
        self.player:addAmmo(100)
        self.player.area:addGameObject('IrregularPolygonEffect', self.player.x, self.player.y, {parent = self.player, w = 8, color = ammo_color})
    end

    if self.rolls.regain_hp_on_kill_chance:next() then
        self.player:addHP(1)
        self.player.area:addGameObject('IrregularPolygonEffect', self.player.x, self.player.y, {parent = self.player, w = 4, color = hp_color})
    end

    if self.rolls.spawn_boost_on_kill_chance:next() then
        self.player.area:addGameObject('Boost', 0, 0, {x = enemy.x, y = enemy.y})
    end

    if self.rolls.spawn_hp_on_kill_chance:next() then
        self.player.area:addGameObject('HP', 0, 0, {x = enemy.x, y = enemy.y})
    end

    if self.rolls.spawn_sp_on_kill_chance:next() then
        self.player.area:addGameObject('SkillPoint', 0, 0, {x = enemy.x, y = enemy.y})
    end
end

function PlayerBuild:onAmmoPickup(ammo)
    if self.rolls.launch_homing_projectile_on_ammo_pickup_chance:next() then
        local d = 2.2*self.w
        self.player.area:addGameObject('Projectile', self.player.x + d*math.cos(self.player.r), self.player.y + d*math.sin(self.player.r), 
        {r = self.player.r, homing = true, parent = self.player, color = barrage_color})
        self.player:addBuff(0.2, barrage_color)
    end

    if self.rolls.regain_hp_on_ammo_pickup_chance:next() then
        self.player:addHP(1)
        self.player.area:addGameObject('IrregularPolygonEffect', self.player.x, self.player.y, {parent = self.player, w = 4, color = hp_color})
    end

    if self.rolls.spawn_fast_attacking_area_on_ammo_pickup_chance:next() then
        self.player.area:addGameObject('EffectArea', ammo.x, ammo.y, {player = self.player, color = aspd_color})
    end
end

function PlayerBuild:onBoostPickup(boost)
    
end

function PlayerBuild:onHPPickup(hpo)
    if self.rolls.spawn_fast_attacking_area_on_hp_pickup_chance:next() then
        self.player.area:addGameObject('EffectArea', hpo.x, hpo.y, {player = self.player, color = aspd_color})
    end

    if self.convert_hp_to_sp_if_hp_full then
        if hp == max_hp then
            self.player:addSkillPoint(3)
            self.player.area:addGameObject('IrregularPolygonEffect', self.player.x, self.player.y, {parent = self.player, w = 4, color = skill_point_color})
        end
    end
end

function PlayerBuild:onSPPickup(sp)
    if self.rolls.regain_hp_on_sp_pickup_chance:next() then
        self.player:addHP(1)
        self.player.area:addGameObject('IrregularPolygonEffect', self.player.x, self.player.y, {parent = self.player, w = 4, color = hp_color})
    end

    if self.rolls.spawn_fast_attacking_area_on_sp_pickup_chance:next() then
        self.player.area:addGameObject('EffectArea', sp.x, sp.y, {player = self.player, color = aspd_color})
    end
end

function PlayerBuild:onBoostStart()
    if self.increased_luck_while_boosting then
        self.luck_boosting = true
        self.luck_boosting_id = self.player:addBuff(nil, luck_color)
        self.luck_multiplier = self.luck_multiplier*2
        self:generateChances()
    end

    if self.invulnerability_while_boosting then
        self.player.invincible = true
        self.player.timer:every('invulnerability_while_boosting', 0.04, function() self.player.disappearing = not self.player.disappearing end)
    end

    if self.launch_homing_projectile_while_boosting then
        self.player.timer:every('launch_homing_projectile_while_boosting', 0.2, function()
            if self.rolls.launch_homing_projectile_while_boosting_chance:next() then
                local d = 2.2*self.player.w
                self.player.area:addGameObject('Projectile', self.player.x + d*math.cos(self.player.r), self.player.y + d*math.sin(self.player.r), 
                {r = self.player.r, homing = true, parent = self.player, color = barrage_color})
                self.player:addBuff(0.2, barrage_color)
            end
        end)
    end
end

function PlayerBuild:onBoostEnd()
    if self.increased_luck_while_boosting and self.luck_boosting then
        self.luck_boosting = false
        self.player:removeBuff(self.luck_boosting_id)
        self.luck_multiplier = self.luck_multiplier/2
        self:generateChances()
    end

    if self.invulnerability_while_boosting then
        self.player.invincible = false
        self.player.disappearing = false
        self.player.timer:cancel('invulnerability_while_boosting')
    end

    if self.launch_homing_projectile_while_boosting then
        self.player.timer:cancel('launch_homing_projectile_while_boosting')
    end
end

return PlayerBuild
