local Projectile = GameObject:extend()

function Projectile:new(area, x, y, opts)
    Projectile.super.new(self, area, x, y, opts)

    self.s = opts.s or 2.5
    self.additional_projectile_size = (self.parent and self.parent.build.additional_projectile_size) or 0

    self:addPhysics(x, y, {collider_type = 'Circle', r = self.s + self.additional_projectile_size, collision_class = 'Projectile'})
    self.v = self.parent.build.pspd_multiplier.value*(opts.v or 200)
    self.rv = opts.rv or table.random({math.randomp(-2*math.pi, -math.pi), math.randomp(math.pi, 2*math.pi)})
    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))

    self.color = opts.color or {255, 255, 255}
    self.timer:every(0.01, function() 
        self.area:addGameObject('TrailParticle', self.x + math.randomp(-1, 1), self.y + math.randomp(-1, 1), 
        {r = math.randomp(self.s - 1, self.s) + self.additional_projectile_size, d = opts.d or math.randomp(0.08, 0.12), color = self.color}) 
    end)

    if self.spin then self.timer:after(math.randomp(2.4, 3.2), function() self:die() end) end
    if self.blast then self.timer:tween(math.randomp(0.4, 0.6), self, {v = 0}, 'linear', function() self:die() end) end
    if self.flame then self.timer:tween(math.randomp(0.6, 1.0), self, {v = 0}, 'in-out-cubic', function() self:die() end) end
    if self.mine then self.timer:after(math.randomp(8, 12), function() self:die() end) end

    if self.random_degree then 
        self.timer:after(0.2, function()
            self.r = self.r + table.random({-1, 1})*math.pi/6
            self.timer:every('forty_five_degree', 0.25*self.parent.build.angle_change_frequency_multiplier.value, function() 
                self.r = self.r + math.randomp(-math.pi/2, math.pi/2)
            end) 
        end)
    end

    if self.ninety_degree then
        self.timer:after(0.2, function()
            self.ninety_degree_direction = table.random({-1, 1})
            self.r = self.r + self.ninety_degree_direction*math.pi/2
            self.timer:every('ninety_degree_first', 0.25*self.parent.build.angle_change_frequency_multiplier.value, function()
                self.r = self.r - self.ninety_degree_direction*math.pi/2
                self.timer:after('ninety_degree_second', 0.1*self.parent.build.angle_change_frequency_multiplier.value, function()
                    self.r = self.r - self.ninety_degree_direction*math.pi/2
                    self.ninety_degree_direction = -1*self.ninety_degree_direction
                end)
            end)
        end)
    end

    if self.fast_slow then
        local source_v = self.v
        self.timer:tween('fast_slow_first', 0.2*self.parent.build.projectile_acceleration_multiplier.value, self, {v = math.randomp(1.6, 2.4)*source_v}, 'in-out-cubic', function()
            self.timer:tween('fast_slow_second', 0.3*self.parent.build.projectile_deceleration_multiplier.value, self, {v = source_v/math.randomp(1.6, 2.4)}, 'linear')
        end)
    end

    if self.slow_fast then
        local source_v = self.v
        self.timer:tween('slow_fast_first', 0.2*self.parent.build.projectile_deceleration_multiplier.value, self, {v = source_v/math.randomp(1.6, 2.4)}, 'in-out-cubic', function()
            self.timer:tween('slow_fast_second', 0.3*self.parent.build.projectile_acceleration_multiplier.value, self, {v = source_v*math.randomp(1.6, 2.4)}, 'linear')
        end)
    end

    if self.wavy then
        local d = table.random({-1, 1})
        self.timer:tween(0.25, self, {r = self.r + self.parent.build.projectile_waviness_multiplier.value*d*math.pi/8}, 'linear', function() 
            self.timer:tween(0.5, self, {r = self.r - self.parent.build.projectile_waviness_multiplier.value*d*math.pi/4}, 'linear') 
        end)
        self.timer:every(0.75, function()
            self.timer:tween(0.25, self, {r = self.r + self.parent.build.projectile_waviness_multiplier.value*d*math.pi/4}, 'linear', function()
                self.timer:tween(0.5, self, {r = self.r - self.parent.build.projectile_waviness_multiplier.value*d*math.pi/4}, 'linear')
            end)
        end)
    end

    if self.shield then
        self.orbitting = true
        self.orbit_distance = math.randomp(32, 64)
        self.orbit_offset = math.randomp(0, 2*math.pi)
        self.orbit_speed = math.randomp(-6, 6)
    end
end

function Projectile:update(dt)
    Projectile.super.update(self, dt)

    -- Collision
    if self.bounce and self.bounce > 0 then
        if self.x < 0 then 
            self.r = math.pi - self.r 
            self.bounce = self.bounce - 1
        end
        if self.y < 0 then 
            self.r = 2*math.pi - self.r 
            self.bounce = self.bounce - 1
        end
        if self.x > gw then 
            self.r = math.pi - self.r 
            self.bounce = self.bounce - 1
        end
        if self.y > gh then 
            self.r = 2*math.pi - self.r 
            self.bounce = self.bounce - 1
        end

    else
        if self.x < 0 then self:die({wall_split_angle = 0}) end
        if self.y < 0 then self:die({wall_split_angle = math.pi/2}) end
        if self.x > gw then self:die({wall_split_angle = -math.pi}) end
        if self.y > gh then self:die({wall_split_angle = -math.pi/2}) end
    end

    if self.collider:enter('Enemy') then
        local _, collider = self.collider:enter('Enemy')
        local other = collider.user_data
        if other then
            other:hit()
            self:die()
            self.parent.build:onKill(other)
        end
    end

    -- Movement
    if self.spin or self.mine then self.r = self.r + self.rv*dt end
    if self.homing then
        if not self.target then
            local nearby_objects = self.area:fastGameObjectQuery(self.x, self.y, 400)
            nearby_objects = fn.select(nearby_objects, function(k, v) return fn.any(enemies, v.class_name) end)
            table.sort(nearby_objects, function(a, b) return distance(self.x, self.y, a.x, a.y) < distance(self.x, self.y, b.x, b.y) end)
            self.target = table.remove(nearby_objects, love.math.random(1, #nearby_objects))
        end
        if self.target and self.target.dead then self.target = nil end

        if self.target then
            local projectile_heading = Vector(self.body:getLinearVelocity()):normalized()
            local angle = math.atan2(self.target.y - self.y, self.target.x - self.x)
            local to_target_heading = Vector(math.cos(angle), math.sin(angle)):normalized()
            local final_heading = (projectile_heading + 0.1*to_target_heading):normalized()
            self.body:setLinearVelocity(self.v*final_heading.x, self.v*final_heading.y)
        else self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r)) end
    else self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r)) end

    if self.shield and self.orbitting then
        self.body:setPosition(self.parent.x + self.orbit_distance*math.cos(self.orbit_speed*time + self.orbit_offset), self.parent.y + self.orbit_distance*math.sin(self.orbit_speed*time + self.orbit_offset))
    end
end

function Projectile:draw()

end

function Projectile:destroy()
    Projectile.super.destroy(self)
end

function Projectile:die(opts)
    local opts = opts or {}

    self.dead = true
    for i = 1, love.math.random(4, 8) do self.area:addGameObject('ExplodeParticle', self.x, self.y, {r = self.s + self.additional_projectile_size, color = self.color}) end

    if self.split then
        self.area:addGameObject('Projectile', self.x + 10*math.cos(self.r - math.pi/4), self.y + 10*math.sin(self.r - math.pi/4), {r = self.r - math.pi/4, color = self.color, parent = self.parent})
        self.area:addGameObject('Projectile', self.x + 10*math.cos(self.r + math.pi/4), self.y + 10*math.sin(self.r + math.pi/4), {r = self.r + math.pi/4, color = self.color, parent = self.parent})
    end

    if self.split and opts.wall_split_angle then
        self.area:addGameObject('Projectile', self.x + 10*math.cos(opts.wall_split_angle - math.pi/4), self.y + 10*math.sin(opts.wall_split_angle - math.pi/4), 
        {r = opts.wall_split_angle - math.pi/4, color = self.color, parent = self.parent})
        self.area:addGameObject('Projectile', self.x + 10*math.cos(opts.wall_split_angle + math.pi/4), self.y + 10*math.sin(opts.wall_split_angle + math.pi/4), 
        {r = opts.wall_split_angle + math.pi/4, color = self.color, parent = self.parent})
    end

    if self.big then
        local angles = {math.pi/4, 3*math.pi/4, -math.pi/4, -3*math.pi/4}
        for _, angle in ipairs(angles) do self.area:addGameObject('Projectile', self.x + 10*math.cos(angle), self.y + 10*math.sin(angle), {r = angle, color = self.color, parent = self.parent}) end
    end

    if self.explode or self.mine then
        self.area:addGameObject('Explosion', self.x, self.y, {color = self.color})
    end
end

return Projectile
