local Stat = Object:extend()

-- increased 10%, increased 20% = base*(1 + 0.1 + 0.2) = base*1.3
-- decreased 10%, decreased 20% = base*(1 - 0.1 - 0.2) = base*0.7
-- more 10%, more 20% = base*(1)*(1.1)*(1.2) = base*1.32
-- less 10%, less 20% = base*(1)*(0.9)*(0.8) = base*0.72

function Stat:new(base, additive, multiplicative)
    self.class_name = 'Stat'
    self.base = base
    self.additive = additive or 0
    self.multiplicative = multiplicative or 1
    self.value = self.base*(1 + self.additive)*(self.multiplicative)
    self.additives = {}
    self.multiplicatives = {}
end

function Stat:update(dt)
    for _, additive in ipairs(self.additives) do self.additive = self.additive + additive end
    for _, multiplicative in ipairs(self.multiplicatives) do self.multiplicative = self.multiplicative*multiplicative end

    if self.additive >= 0 then
        self.value = self.base*(1 + self.additive)*self.multiplicative
    else
        self.value = (self.base/(1 - self.additive))*self.multiplicative
    end

    self.additive = 0
    self.multiplicative = 1
    self.additives = {}
    self.multiplicatives = {}
end

function Stat:increase(percentage)
    table.insert(self.additives, percentage*0.01)
end

function Stat:decrease(percentage)
    table.insert(self.additives, -percentage*0.01)
end

function Stat:more(percentage)
    table.insert(self.multiplicatives, percentage*0.01)
end

function Stat:less(percentage)
    table.insert(self.multiplicatives, -percentage*0.01)
end

function Stat:set(value)
    self.base = value
end

return Stat
