local Waver = GameObject:extend()

function Waver:new(area, x, y, opts)
    Waver.super.new(self, area, x, y, opts)

    self.w, self.h = 12, 6
    self.x = table.random({0 - 40, gw + 40})
    self.y = math.randomp(40, gh - 40)
    if opts.x and opts.y then self.x, self.y = opts.x, opts.y end
    self.vertices = {self.w, 0, 0, self.h, -self.w, 0, 0, -self.h}
    self:addPhysics(self.x, self.y, {collider_type = 'Polygon', vertices = self.vertices, collision_class = 'Enemy'})
    self.body:setPosition(self.x, self.y)

    self.v = opts.v or math.randomp(90, 110)
    self.r = 0
    if math.sign(self.x) > 0 then self.r = math.pi end
    
    --[[
    self.timer:every(math.randomp(3, 5), function()
        self.area:addGameObject('PreAttackEffect', self.x + 1.4*self.w*math.cos(self.body:getAngle()), self.y + 1.4*self.w*math.sin(self.body:getAngle()), {parent = self, color = enemy_color, duration = 1})
        self.timer:after(1, function()
            self.area:addGameObject('EnemyProjectile', self.x + 1.4*self.w*math.cos(self.body:getAngle()), self.y + 1.4*self.w*math.sin(self.body:getAngle()), 
            {r = self.body:getAngle(), v = math.randomp(140, 160)})
        end)
    end)
    ]]--

    self.timer:every(math.randomp(1.4, 2.2), function()
        self.timer:after(1, function()
            self.area:addGameObject('EnemyProjectile', self.x + 1.4*self.w*math.cos(self.body:getAngle()), self.y + 1.4*self.w*math.sin(self.body:getAngle()), 
            {r = self.body:getAngle(), v = math.randomp(140, 160)})
            self.area:addGameObject('EnemyProjectile', self.x + 1.4*self.w*math.cos(self.body:getAngle() - math.pi), self.y + 1.4*self.w*math.sin(self.body:getAngle() - math.pi), 
            {r = self.body:getAngle() - math.pi, v = math.randomp(140, 160)})
        end)
    end)

    local d = table.random({-1, 1})
    local m = math.randomp(1, 4)
    self.timer:tween(0.25, self, {r = self.r + m*d*math.pi/8}, 'linear', function() 
        self.timer:tween(0.5, self, {r = self.r - m*d*math.pi/4}, 'linear') 
    end)
    self.timer:every(0.75, function()
        self.timer:tween(0.25, self, {r = self.r + m*d*math.pi/4}, 'linear', function()
            self.timer:tween(0.5, self, {r = self.r - m*d*math.pi/4}, 'linear')
        end)
    end)
end

function Waver:update(dt)
    Waver.super.update(self, dt)

    self.body:setLinearVelocity(self.v*math.cos(self.r), self.v*math.sin(self.r))
    self.body:setAngle(self.r)
    if self.y < -40 then self:hit() end
    if self.y > gh + 40 then self:hit() end
end

function Waver:draw()
    love.graphics.setColor(enemy_color)
    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    for i = 1, #points, 2 do
        if i < #points-2 then 
            love.graphics.line(points[i], points[i+1], points[i+2], points[i+3]) 
        elseif i >= #points-2 then
            love.graphics.line(points[i], points[i+1], points[1], points[2]) 
        end
    end
    love.graphics.setColor(255, 255, 255)
end

function Waver:destroy()
    Waver.super.destroy(self)
end

function Waver:hit()
    if self.dead then return end
    self.dead = true

    self.area:addGameObject('Ammo', self.x, self.y)
    score = score + 300

    local points = {self.body:getWorldPoints(self.collider.shapes.main:getPoints())}
    local cx, cy = Math.polygon.getCentroid(points)
    for i = 1, #points, 2 do
        if i < #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[i+2], points[i+3]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = enemy_color})
        elseif i >= #points-2 then 
            local x1, y1, x2, y2 = points[i], points[i+1], points[1], points[2]
            local vx, vy = self.body:getLinearVelocityFromWorldPoint((x1+x2)/2, (y1+y2)/2)
            self.area:addGameObject('DeadLine', cx, cy, {x1 = x1, y1 = y1, x2 = x2, y2 = y2, vx = vx, vy = vy, color = enemy_color})
        end
    end
end

return Waver
