local Director = Object:extend()

function Director:new(stage)
    self.timer = Timer()
    self.stage = stage
    self.area = self.stage.area

    self.difficulty_to_points = {
        [1] = 18,
        [2] = 18,
        [3] = 24,
        [4] = 18,
        [5] = 32,
        [6] = 24,
        [7] = 40,
        [8] = 32,
        [9] = 48,
        [10] = 42,
        [11] = 16,
        [12] = 32,
        [13] = 64,
        [14] = 64,
        [15] = 48,
    }

    self.difficulty_plus_to_points = {
        [1] = 20,
        [2] = 20,
        [3] = 32,
        [4] = 20,
        [5] = 40,
        [6] = 32,
        [7] = 48,
        [8] = 40,
        [9] = 56,
        [10] = 56,
        [11] = 32,
        [12] = 40,
        [13] = 80,
        [14] = 80,
        [15] = 56,
    }

    self.enemy_spawn_chances = {
        [1] = chanceList({'Rock', 12}, {'BigRock', 2}), 
        [2] = chanceList({'Rock', 2}, {'Shooter', 1}), 
        [3] = chanceList({'Rock', 12}, {'BigRock', 6}, {'Shooter', 6}, {'Seeker', 6}), 
        [4] = chanceList({'Rock', 1}), 
        [5] = chanceList({'Rock', 8}, {'BigRock', 8}, {'Shooter', 8}), 
        [6] = chanceList({'Rock', 5}, {'Waver', 1}, {'Seeker', 1}, {'Shooter', 1}), 
        [7] = chanceList({'Rock', 24}, {'BigRock', 8}, {'Waver', 8}, {'Shooter', 8}), 
        [8] = chanceList({'Rock', 12}, {'BigRock', 4}, {'Waver', 6}, {'Shooter', 8}), 
        [9] = chanceList({'Rock', 8}, {'BigRock', 4}, {'Waver', 4}, {'Shooter', 4}, {'Seeker', 4}), 
        [10] = chanceList({'Rock', 8}, {'BigRock', 4}, {'Waver', 4}, {'Shooter', 4}, {'Seeker', 4}), 
        [11] = chanceList({'Orbitter', 2}), 
        [12] = chanceList({'Rock', 4}, {'BigRock', 4}, {'Waver', 4}, {'Shooter', 4}, {'Seeker', 4}, {'Orbitter', 4}), 
        [13] = chanceList({'Rock', 12}, {'BigRock', 4}, {'Waver', 4}, {'Shooter', 4}, {'Seeker', 4}, {'Orbitter', 4}), 
        [14] = chanceList({'Rock', 12}, {'BigRock', 4}, {'Waver', 4}, {'Shooter', 4}, {'Seeker', 4}, {'Orbitter', 4}), 
        [15] = chanceList({'Rock', 12}, {'BigRock', 4}, {'Waver', 4}, {'Shooter', 4}, {'Seeker', 4}, {'Orbitter', 4}), 
    }

    self.resource_spawn_chances = chanceList({'Boost', 4}, {'HP', 2}, {'SkillPoint', 8})

    self.enemy_to_points = {
        ['Rock'] = 1,
        ['BigRock'] = 2,
        ['Shooter'] = 2,
        ['Waver'] = 4,
        ['Seeker'] = 6,
        ['Orbitter'] = 8,
    }

    self.difficulty = 1
    self.round_duration = 22 
    self.round_timer = 0
    self.resource_duration = 16
    self.resource_timer = 0
    self.attack_duration = 30
    self.attack_timer = 0
    self:startRound()

    self.area:addGameObject('Attack', 0, 0)
end

function Director:update(dt)
    self.timer:update(dt)

    self.round_timer = self.round_timer + dt
    if self.round_timer > self.round_duration then
        self.round_timer = 0
        self.difficulty = self.difficulty + 1
        self:startRound()
    end

    self.resource_timer = self.resource_timer + dt
    if self.resource_timer > self.resource_duration then
        self.resource_timer = 0
        self.area:addGameObject(self.resource_spawn_chances:next())
    end

    self.attack_timer = self.attack_timer + dt
    if self.attack_timer > self.attack_duration then
        self.attack_timer = 0
        self.area:addGameObject('Attack', 0, 0)
    end
end

function Director:draw()
    -- love.graphics.print(self.difficulty, gw - 20, gh - 20)
end

function Director:startRound()
    local points = self.difficulty_to_points[self.difficulty]
    local runs = 0

    -- Find enemies
    self.enemy_list = {}
    while points > 0 and runs < 100 do
        local enemy = self.enemy_spawn_chances[self.difficulty]:next()
        points = points - self.enemy_to_points[enemy]
        table.insert(self.enemy_list, enemy)
        runs = runs + 1
    end

    -- Find enemies spawn times
    self.enemy_spawn_times = {}
    for i = 1, #self.enemy_list do self.enemy_spawn_times[i] = math.randomp(0, self.round_duration) end
    table.sort(self.enemy_spawn_times, function(a, b) return a < b end)

    -- Set spawn enemy timer
    for i = 1, #self.enemy_spawn_times do
        self.timer:after(self.enemy_spawn_times[i], function()
            self.area:addGameObject(self.enemy_list[i])
        end)
    end
end

return Director
