local Stage = Object:extend()

function Stage:new()
    self.id = getUID()
    self.area = Area(self)

    font = fonts.TinyUnicode_16
    love.graphics.setFont(font)

    self.director = Director(self)

    self.main_canvas = love.graphics.newCanvas(gw, gh)
    self.main_canvas:setFilter(filter_mode, filter_mode, anisotropy)

    self.player = self.area:addGameObject('Player', gw/2, gh/2)
    self.show_tutorial = true
    self.tutorial_alpha = 192
    timer:after(4, function() 
        timer:tween(1, self, {tutorial_alpha = 0}, 'in-out-cubic', function() self.show_tutorial = false end)
    end)
end

function Stage:finish()
    self.end_stage = true
    table.insert(session_data.events, {score = score, difficulty = self.director.difficulty})
end

function Stage:update(dt)
    camera.smoother = Camera.smooth.damped(5)
    camera:lockPosition(dt, gw/2, gh/2)

    self.area:update(dt)
    self.director:update(dt)

    if self.end_stage then
        if input:pressed('down') or input:pressed('up') then
            gameRestart()
        end
    end
end

function Stage:draw()
    love.graphics.setCanvas(self.main_canvas)
    love.graphics.clear()
        love.graphics.setFont(font)

        -- Tutorial
        if self.show_tutorial then
            love.graphics.setColor(255, 255, 255, self.tutorial_alpha)
            love.graphics.rectangle('line', gw/2 - 6, gh/2 - 6, 12, 12)
            love.graphics.rectangle('line', gw/2 - 6, gh/2 - 6 - 16, 12, 12)
            love.graphics.rectangle('line', gw/2 - 6 - 16, gh/2 - 6, 12, 12)
            love.graphics.rectangle('line', gw/2 - 6 + 16, gh/2 - 6, 12, 12)

            pushRotate(gw/2 - 0.5, gh/2 - 1, math.pi)
            draft:triangleEquilateral(gw/2 - 0.5, gh/2 - 1, 5, 'fill')
            love.graphics.pop()
            draft:triangleEquilateral(gw/2 - 0.5, gh/2 - 1 - 16, 5, 'fill')
            pushRotate(gw/2 - 0.5 - 16, gh/2 - 1, -math.pi/2)
            draft:triangleEquilateral(gw/2 - 0.5 - 16, gh/2 - 1, 5, 'fill')
            love.graphics.pop()
            pushRotate(gw/2 - 0.5 + 16, gh/2 - 1, math.pi/2)
            draft:triangleEquilateral(gw/2 - 0.5 + 16, gh/2 - 1, 5, 'fill')
            love.graphics.pop()
            love.graphics.setColor(255, 255, 255, 255)
        end
        
        camera:attach(0, 0, gw, gh)
        self.area:draw()
        camera:detach()

        -- HP
        love.graphics.setFont(font)
        love.graphics.setColor(255, 255, 255)
        love.graphics.setColor(hp_color)
        local health_w = 4
        local spacing = 8
        local w = (max_hp-1)*spacing + max_hp*health_w
        for i = 1, max_hp do love.graphics.circle('line', gw/2 - w/2 + (i-1)*(health_w+spacing), gh - 32, health_w) end
        for i = 1, hp do love.graphics.circle('fill', gw/2 - w/2 + (i-1)*(health_w+spacing), gh - 32, health_w) end
        love.graphics.setColor(255, 255, 255)

        -- Ammo
        love.graphics.setColor(255, 255, 255)
        love.graphics.setColor(ammo_color)
        love.graphics.rectangle('line', gw/2 - 32, gh - 24, 64, 4)
        love.graphics.rectangle('fill', gw/2 - 32, gh - 24, 64*(ammo/max_ammo), 4)
        love.graphics.setColor(255, 255, 255)

        -- Boost
        love.graphics.setColor(255, 255, 255)
        love.graphics.setColor(boost_color)
        love.graphics.rectangle('line', gw/2 - 32, gh - 16, 64, 4)
        love.graphics.rectangle('fill', gw/2 - 32, gh - 16, 64*(boost/max_boost), 4)
        love.graphics.setColor(255, 255, 255)

        -- Skill Point
        love.graphics.setColor(255, 255, 255)
        love.graphics.setColor(skill_point_color)
        love.graphics.print(math.roundp(skill_point, 0), 10, 10, 0, 1, 1, 0, font:getHeight()/2)
        love.graphics.setColor(255, 255, 255)

        -- Score
        love.graphics.setColor(255, 255, 255)
        love.graphics.print(math.roundp(score, 0), gw - 20, 10, 0, 1, 1, math.floor(font:getWidth(math.roundp(score, 0))/2), font:getHeight()/2)
        love.graphics.setColor(255, 255, 255)

        -- Director
        self.director:draw()

        -- Score Screen
        if self.end_stage then
            love.graphics.setFont(font)
            love.graphics.print('SCORE: ' .. score, gw/2, gh/2 - 16, 0, 1, 1, math.floor(font:getWidth('SCORE: ' .. score)/2), math.floor(font:getHeight()/2))
            love.graphics.print('PRESS BOOST TO RETRY', gw/2, gh/2 + 16, 0, 1, 1, math.floor(font:getWidth('PRESS BOOST TO RETRY')/2), math.floor(font:getHeight()/2))
        end
    love.graphics.setCanvas()

    love.graphics.setColor(255, 255, 255)
    love.graphics.setBlendMode("alpha", "premultiplied")
    love.graphics.draw(self.main_canvas, 0, 0, 0, sx, sy)
    love.graphics.setBlendMode("alpha")
end

return Stage
